/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package carto;

import java.awt.Font;
import java.awt.geom.Point2D;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Date;
import java.util.Vector;

import donnees.TableModel;

/**
 * 
 * @author Administrateur
 */
public class Carte implements Cloneable, Serializable {

	public Vector c;

    public TableModel tm;

    private int typeAnalyse; // 1 = plages, 2 = symboles, 3 = plages sous, 4 = plages dans

    private int typeEchelle;

    private Point2D coordMin;

    private Point2D coordMax;

    private String titre;

    private Font policeTitre;

    private String sousTitre;

    private Font policeSousTitre;

    private Font policeLegende;

    private Date dateSource;

    private Font policeDate;

    private Boolean titreAuto;

    public AnalyseSymboles analyseSymb;

    public AnalysePlages analysePlages;

    /**
     * @return Returns the policeLegende.
     */
    public Font getPoliceLegende() {
        return policeLegende;
    }

    /**
     * @param policeLegende
     *            The policeLegende to set.
     */
    public void setPoliceLegende(Font policeLegende) {
        this.policeLegende = policeLegende;
    }

    /** Creates a new instance of Carte */
    public Carte() {
        int i = 0;
        analysePlages = new AnalysePlages();
        analysePlages.nbreClasses = 8;
        for (i = 0; i < 8; i++) {
            ClasseDiscret cl = new ClasseDiscret();
            analysePlages.classes.add(cl.clone());
        }
        analyseSymb = new AnalyseSymboles();
    }

    public void setTypeAnalyse(int valeur) {
        this.typeAnalyse = valeur;
    }

    public int getTypeAnalyse() {
        return this.typeAnalyse;
    }

    public void setTypeEchelle(int valeur) {
        this.typeEchelle = valeur;
    }

    public int getTypeEchelle() {
        return this.typeEchelle;
    }

    public Object clone() {
        try {
            Carte c2 = null;
            c2 = (Carte) super.clone();
            //c2.typeAnalyse = this.typeAnalyse;
            return c2;
        } catch (CloneNotSupportedException x) {
            return (new InternalError(Messages.getString("Carte.0"))); //$NON-NLS-1$
        }
    }
    
    public Carte copie() {
    	Carte c2 = new Carte();
    	AnalysePlages ap2 = new AnalysePlages();
    	ap2 = this.analysePlages.copie();
    	c2.analysePlages = ap2;
    	AnalyseSymboles as2 = new AnalyseSymboles();
    	as2 = this.analyseSymb.copie();
    	c2.analyseSymb = as2;
    	c2.typeAnalyse = this.typeAnalyse;
    	c2.typeEchelle = this.typeEchelle;
    	c2.coordMax = this.coordMax;
    	c2.coordMin = this.coordMin;
    	c2.dateSource = this.dateSource;
    	c2.policeDate = this.policeDate;
    	c2.policeLegende = this.policeLegende;
    	c2.policeTitre = this.policeTitre;
    	c2.policeSousTitre = this.policeSousTitre;
    	c2.titre = this.titre;
    	c2.sousTitre = this.sousTitre;
    	return c2;
    }
    
    public void recyclage() {
        this.setTypeAnalyse(0) ;
        if (this.analysePlages != null) {
        	this.analysePlages.videClasses();
        	this.analysePlages.champNum = -1;
        }
        if (this.analyseSymb != null) {
            this.analyseSymb.vide();
            this.analyseSymb.champNum = -1;
        }	
    }

    public void enregistrerCarte(String fichier) {
        try {
            FileOutputStream sortie = new FileOutputStream(fichier);
            ObjectOutputStream oo = new ObjectOutputStream(sortie);
            oo.writeObject(this);
            oo.flush();
            oo.close();
        } catch (IOException e) {
            System.out.println(Messages.getString("Carte.1")); //$NON-NLS-1$
        }
    }
}