/*
 * 18 mars 2005 10h00
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package carto;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

import donnees.champDonnees;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.io.WKTReader;

public class FondCarte implements Serializable {     // le fond de carte

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected Carte carteEnCours;

	private double[] extremes;

	private double coefMEP = 1;

	private Vector colonnes;

	private Vector ensMetaPoly; // Vector de MetaPolygones utilisé pour dessiner

	private double hauteur; // hauteur du fond de carte

	private double largeur; // largeur du fond de carte

	protected ParamsPolygone params;

	protected int[] posDec;

	private boolean pourMEP = false;

	protected int PrecedentChamp;

	private Vector tableau;

	private String uniteCoord;

	protected boolean[] vNul;

	private double Xmin;

	private double Ymax;

	private int pSel = -1;

	private int nbPts = 0;

	/** Creates a new instance of FondCarte */
	public FondCarte() {
		this.ensMetaPoly = new Vector();
	}

	public FondCarte(String action, String fichier) {
		this.ensMetaPoly = new Vector();
		// soit on importe le fond de carte (action = "importation") soit on
		// ouvre un fond de carte existant (action = "ouverture")
		if (action.equals("importation")) { //$NON-NLS-1$
			this.importerFondMIF(fichier);
			this.carteEnCours = new Carte();
			this.calculTailleCarte();
			this.decalerCoord();
		} else { //ouverture
			this.ouvrirFond(fichier);
			this.carteEnCours = new Carte();
			this.calculTailleCarte();
			// pas besoin de décaler les coordonnées dans le fichier elles sont
			// déjé decalées
		}
	}

	/* débug : affichage des coordonnées du fond de carte */
	public void afficherCoord() {
		MetaPolygone mp;

		System.out.println(Messages.getString("FondCarte.23")); //$NON-NLS-1$

		if (this.ensMetaPoly != null) {
			for (int i = 0; i < this.ensMetaPoly.size(); i++) {
				mp = (MetaPolygone) this.ensMetaPoly.elementAt(i);
				mp.afficherCoord();
			}
		} else {
			System.out.println(Messages.getString("FondCarte.24")); //$NON-NLS-1$
		}
	}

	/*
	 * fonction qui permet d'ajouter un MetaPolygone
	 */
	public void ajouterMetaPoly(MetaPolygone m) {
		this.ensMetaPoly.addElement(m);
	}

	/*
	 * public void setEnsMPOrigine(Vector v){ this.ensMPOrigine = v; }
	 */

	/*
	 * fonction qui va renvoyer la position dans le tableau du polygone auquel
	 * le point p appartient. Renvoie -1 si il nappartient é aucun metapolygone
	 * de la carte
	 */
	public int appartenir(Point2D p) {
		int i = 0;
		boolean appartient = false;

		while (i < this.ensMetaPoly.size() && appartient == false) {
			appartient = ((MetaPolygone) this.ensMetaPoly.get(i)).appartenir(p);
			i++;
		}
		if (appartient == false)
			return -1;
		else
			return i - 1;
	}

	/* fonction qui permet de redimensionner le fond de carte */
	public void calculerCoord(int larg, int haut, double coeff) {
		MetaPolygone mPoly;
		for (int i = 0; i < this.ensMetaPoly.size(); i++) {
			((MetaPolygone) this.ensMetaPoly.elementAt(i)).calculerCoord(larg, haut, coeff);
		}
	}

	/*
	 * fonction qui permet de calculer la largeur et la hauteur (attributs de
	 * classe) du fond de carte
	 */
	public void calculTailleCarte() {
		this.extremes = new double[4];
		this.extremes = this.trouverExtremes();
		this.largeur = (extremes[2] - extremes[0]);
		this.hauteur = (extremes[1] - extremes[3]);
	}

	/*
	 * fonction qui recalcule les coordonnées pour les adapter au systeme de
	 * coordonnés Java (origine en haut é droite). Cette fonction est utile dans
	 * l'importaion au format Mif. Dans le format d'enregistrement dees donnnées
	 * actuel les données qui sont enregistrées sont déjé adaptées é ce systeme
	 * de coordonnées.
	 */
	public void decalerCoord() {
		MetaPolygone mPoly;

		for (int i = 0; i < this.ensMetaPoly.size(); i++) {
			mPoly = (MetaPolygone) this.ensMetaPoly.get(i);
			mPoly.decalerCoord(this.Xmin, this.Ymax);
			this.ensMetaPoly.setElementAt(mPoly, i);
		}
	}

	public ParamsPolygone definitionParams(int index) {
		int i = 0;
		//Définition des paramétres carto du metapolygone index selon l'analyse
		// é dessiner
		ParamsPolygone p = new ParamsPolygone();
		if (carteEnCours.getTypeAnalyse() != 2) {
			//Si choropléthe
			double valeur = ((Double) carteEnCours.tm.getValueAt(index, carteEnCours.analysePlages.champNum)).doubleValue();
			for (i = 0; i < carteEnCours.analysePlages.nbreClasses; i++) {
				if (i < (carteEnCours.analysePlages.nbreClasses - 1)) {
					if (valeur > ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).limiteInf
							&& valeur < ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).limiteSup) {
						p.couleur = ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).couleur;
					}
				} else {
					if (valeur > ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).limiteInf
							&& valeur <= ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).limiteSup) {
						p.couleur = ((ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(i)).couleur;
					}
				}
			}
		}
		return p;
	}

	/*
	 * Fonction permettant de dessiner le fond de carte
	 */
	public void drawFondCarte(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		Stroke origineStroke = g2.getStroke();
		//Paint originePaint = g2.getPaint();

		boolean type4 = (carteEnCours.getTypeAnalyse() == 4);
		if (this.ensMetaPoly != null) {
			for (int i = 0; i < ensMetaPoly.size(); i++) {
				if (this.pSel == i) {
					g2.setStroke(new BasicStroke(5f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
				} else {
					if (g2.getStroke() != origineStroke) g2.setStroke(origineStroke);
				}
				
				((MetaPolygone) this.ensMetaPoly.elementAt(i)).drawMetaPolygone(g, type4);
			}
		}

		if (this.ensMetaPoly != null && carteEnCours.getTypeAnalyse() != 1 && carteEnCours.getTypeAnalyse() != 0) {
			Color cSurface;
			Color cContour;
			Color tColor;
			float largeur;
			for (int i = 0; i < ensMetaPoly.size(); i++) {
				largeur = 3f;
				Point centre = ((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getCentroide();
				int rayon;
				if (!pourMEP)
					rayon = (int) (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().tailleSymbole * carteEnCours.analyseSymb.cS);
				else
					rayon = (int) (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().tailleSymbole * coefMEP);
				if (rayon == 0) continue;
				if (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().negatif == false) {
					if (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().estEvide == false) {
						cSurface = carteEnCours.analyseSymb.couleurSymbole;
						cContour = carteEnCours.analyseSymb.couleurContour;
					} else {
						cSurface = new Color(0, 0, 0, 1); //carteEnCours.analyseSymb.couleurSymbole;
						cContour = carteEnCours.analyseSymb.couleurSymbole; //carteEnCours.analyseSymb.couleurContour;
					}
				} else {
					if (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().estEvide == false) {
						cSurface = carteEnCours.analyseSymb.couleurSymboleNeg;
						cContour = carteEnCours.analyseSymb.couleurContourNeg;                		
					}
					else {                   
						cSurface = new Color(0, 0, 0, 1); //carteEnCours.analyseSymb.couleurSymbole;
						cContour = carteEnCours.analyseSymb.couleurSymboleNeg; //carteEnCours.analyseSymb.couleurContour;
					}
				}
				if (carteEnCours.getTypeAnalyse() == 4) cSurface = ((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().couleur;
				if (carteEnCours.analyseSymb.typeSymbole == 0) {
					if (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().estEvide == false) {
						tColor = new Color(cSurface.getRed(), cSurface.getGreen(), cSurface.getBlue(), (int)(carteEnCours.analyseSymb.opacite * 255));
						largeur = 1.2f;
						g2.setPaint(tColor);
						g2.fillOval((int) (centre.x - (rayon / 2)), (int) (centre.y - (rayon / 2)), rayon, rayon);
					}
					//g2.setStroke(new BasicStroke(largeur, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
					g2.setPaint(cContour);
					g2.drawOval((int) (centre.x - (rayon / 2)), (int) (centre.y - (rayon / 2)), rayon, rayon);
				} else {
					if (((MetaPolygone) this.ensMetaPoly.elementAt(posDec[i])).getParams().estEvide == false) {
						tColor = new Color(cSurface.getRed(), cSurface.getGreen(), cSurface.getBlue(), (int)(carteEnCours.analyseSymb.opacite * 255));
						largeur = 1.2f;
						g2.setPaint(tColor);
						g2.fillRect((int) (centre.x - (rayon / 2)), (int) (centre.y - (rayon / 2)), rayon, rayon);
					}
					//g2.setStroke(new BasicStroke(largeur, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
					g2.setPaint(cContour);
					g2.drawRect((int) (centre.x - (rayon / 2)), (int) (centre.y - (rayon / 2)), rayon, rayon);
				}
			}
		}

		/*
		 * g.setColor(new Color(0,255,255)); for (int i=0;i
		 * <ensMPOrigine.size();i++){ ((MetaPolygone)
		 * this.ensMPOrigine.elementAt(i)).drawMetaPolygone(g); }
		 */
	}

	/**
	 * * enregistrement par sérialisation : trés rapide é l'enregistrement mais
	 * trés long é l'ouverture public void enregistrerFond1(String fichier){
	 * 
	 * try{ FileOutputStream flotSortie = new FileOutputStream(fichier);
	 * ObjectOutputStream flotDonnees = new ObjectOutputStream(flotSortie);
	 * 
	 * flotDonnees.writeObject(this); flotDonnees.close(); } catch(Exception
	 * err){ err.printStackTrace(); } }
	 */

	/*
	 * Fonction d'enregistrement du fond de carte Enregistre le fond de carte au
	 * format texte
	 */
	public void enregistrerFond(String fichier) {
		try {
			FileWriter sortie = new FileWriter(fichier);
			BufferedWriter tampon = new BufferedWriter(sortie);
			tampon.write("#" + uniteCoord);
			tampon.newLine();
			for (int i = 0; i < this.ensMetaPoly.size(); i++) {
				((MetaPolygone) this.ensMetaPoly.get(i)).enregistrerMP(tampon);
			}
			tampon.close();
			sortie.close();
		} catch (IOException e) {
			System.out.println(Messages.getString("FondCarte.25")); //$NON-NLS-1$
		}
	}

	public MetaPolygone findPolygoneAt(Point2D pt) {
		for (int i = 0; i < this.ensMetaPoly.size(); i++) {
			if (((MetaPolygone)this.ensMetaPoly.elementAt(i)).appartenir(pt))
				return (MetaPolygone)this.ensMetaPoly.elementAt(i);
		}
		return null;
	}

	public Carte getCarte() {
		return this.carteEnCours;
	}

	/*
	 * Fonction permettant de retourner une structure ClasseDiscret indexée
	 */
	private ClasseDiscret getClasse(int index) {
		return (ClasseDiscret) carteEnCours.analysePlages.classes.elementAt(index);
	}

	public double getCoefMEP() {
		return coefMEP;
	}

	public Vector getEnsMetaPoly() {
		return this.ensMetaPoly;
	}

	public double getHauteur() {
		return this.hauteur;
	}

	/* Méthodes d'accés é la largeur et la hauteur de la carte */
	public double getLargeur() {
		return this.largeur;
	}

	public Vector getTableau() {
		return this.tableau;
	}

	public String getUniteCoord() {
		return uniteCoord;
	}

	/* fonction qui définit un fond de carte pour tester l'affichage */
	/*
	 * public void test (){
	 * 
	 * Point2D.Double p1 = new Point2D.Double(0.,0.); Point2D.Double p2 = new
	 * Point2D.Double(0.,100.); Point2D.Double p3 = new
	 * Point2D.Double(100.,100.); Point2D.Double p4 = new
	 * Point2D.Double(100.0,0.0); Point2D.Double p5 = new
	 * Point2D.Double(150.0,100.0);
	 * 
	 * Point2D.Double p6 = new Point2D.Double(100.0,200.0); Point2D.Double p7 =
	 * new Point2D.Double(200.0,200.0); Point2D.Double p8 = new
	 * Point2D.Double(200.0,100.0);
	 * 
	 * Polygone poly1 = new Polygone(); Polygone poly2 = new Polygone();
	 * 
	 * poly1.ajouterPoint(p1); poly1.ajouterPoint(p2); poly1.ajouterPoint(p3);
	 * poly1.ajouterPoint(p4); //poly1.ajouterPoint(p5); poly1.ajouterPoint(p1);
	 * 
	 * poly2.ajouterPoint(p3); poly2.ajouterPoint(p6); //poly2.ajouterPoint(p6);
	 * poly2.ajouterPoint(p7); poly2.ajouterPoint(p8); poly2.ajouterPoint(p3);
	 * 
	 * MetaPolygone m1 = new MetaPolygone(); MetaPolygone m2 = new
	 * MetaPolygone();
	 * 
	 * m1.ajouterPolygone(poly1); m2.ajouterPolygone(poly2);
	 * 
	 * this.ajouterMetaPoly(m1); this.ajouterMetaPoly(m2); }
	 */

	/*
	 * Methode pour importer un fond de format MIF. Recupére le fond de carte
	 * contenu dans le fichier, crée et instancie l'objet FondCarte
	 * correspondant.
	 */
	public void importerFondMIF(String fichier) {

		try {
			String fichierMif = fichier;
			String fichierMid = fichier.substring(0, fichier.length() - 4) + ".MID"; //$NON-NLS-1$
			String ligneMif;
			String ligneMid;
			String delimiteur;

			int nbPoly;
			int nbPoints;
			int positionCar;
			int positionCar2;
			int nbColonnes = 0;

			double X;
			double Y;
			double centreX;
			double centreY;

			boolean centrePresent;

			Vector donnees = new Vector(); // contient les données du fond de
			// carte
			// Vector é deux dimensions: chaque case
			// du Vector contient un Vector qui
			// correspond é un champs de données

			Vector champ; // Vector qui corespond é un champ de donnée la
			// premiére case corespondant au nom du champ de données

			Polygone poly;
			MetaPolygone mPoly;

			FileReader entreeMif = new FileReader(fichierMif);
			BufferedReader tamponMif = new BufferedReader(entreeMif);

			//Verification que le fichier MIF n'est pas vide
			String debut = tamponMif.readLine().subSequence(0, 7).toString().toUpperCase();
			if (!debut.equals("VERSION")) //$NON-NLS-1$
				System.out.println(Messages.getString("FondCarte.3")); //$NON-NLS-1$
			else {

				//recherche de la ligne contenant le delimiteur
				ligneMif = tamponMif.readLine();
				String delim = ligneMif.subSequence(0, 9).toString().toUpperCase();
				while (!delim.equals("DELIMITER")) { //$NON-NLS-1$
					ligneMif = tamponMif.readLine();
					delim = ligneMif.subSequence(0, 9).toString().toUpperCase();
				}

				positionCar = ligneMif.indexOf(" "); //$NON-NLS-1$
				delimiteur = ligneMif.substring(positionCar + 2, positionCar + 3);
				//System.out.println(delimiteur); //debug

				//Récupération de l'unité des coordonnées
				ligneMif = tamponMif.readLine();
				if (ligneMif.length() >= 8) {
					delim = ligneMif.subSequence(0, 8).toString().toUpperCase();
				} else
					delim = ligneMif;
				while (!delim.equals("COORDSYS")) { //$NON-NLS-1$
					ligneMif = tamponMif.readLine();
					if (ligneMif.length() >= 8) {
						delim = ligneMif.subSequence(0, 8).toString().toUpperCase();
					} else
						delim = ligneMif;
				}
				if (ligneMif.indexOf("m") == -1) {
					if (ligneMif.indexOf("km") == -1) {
						this.uniteCoord = "inconnu";
					} else {
						this.uniteCoord = "km";
					}
				} else {
					this.uniteCoord = "m";
				}				

				//tant que l'on a pas atteint la ligne Columns on avance
				ligneMif = tamponMif.readLine();
				delim = ligneMif.subSequence(0, 7).toString().toUpperCase();
				while (!delim.equals("COLUMNS")) { //$NON-NLS-1$
					ligneMif = tamponMif.readLine();
					delim = ligneMif.subSequence(0, 7).toString().toUpperCase();
				}

				// les deux lignes qui suivent correspondent aux colonnes
				// geocode et nom
				// on les passe car on ne les met pas dans le Vector donnees mis
				// directement
				// dans la structure MetaPolygone
				ligneMif = tamponMif.readLine();
				if (ligneMif.equals("data") || ligneMif.equals("DATA")) //$NON-NLS-1$
					System.out.println(Messages.getString("FondCarte.8")); //$NON-NLS-1$
				else {
					ligneMif = tamponMif.readLine();
					if (ligneMif.equals("data")  || ligneMif.equals("DATA")) //$NON-NLS-1$
						System.out.println(Messages.getString("FondCarte.10")); //$NON-NLS-1$
					else {

						// tant que l'on a pas atteint la ligne Data on avance
						// et on met dans la premiére colonne des Vecteurs
						// correspondant aux
						// champs de données le nom de la colonne de données
						ligneMif = tamponMif.readLine();
						while (!ligneMif.toUpperCase().equals("DATA") && ligneMif.length() != 0) { //$NON-NLS-1$
							champ = new Vector();
							//while (ligneMif.startsWith(" ")) { //$NON-NLS-1$
							//	ligneMif = ligneMif.substring(1);
							//}
							ligneMif = ligneMif.trim();
							positionCar = ligneMif.indexOf(" "); //$NON-NLS-1$
							champ.add(ligneMif.substring(0, positionCar));
							donnees.add(champ);
							ligneMif = tamponMif.readLine();
						}

						// System.out.println(ligne); //debug

						//tamponMif.readLine(); //on avance d'une ligne (ligne
						// vide)

						FileReader entreeMid = new FileReader(fichierMid);
						BufferedReader tamponMid = new BufferedReader(entreeMid);

						ligneMid = tamponMid.readLine();// permiére ligne du
						// fichier

						// Si le fichier ne contient pas le délimiteur ce n'est
						// pas un fichier MID
						// ou alors ce fichier ne contient qu'un seul champ de
						// données donc il ne
						// peut pas étre importé
						if (ligneMid.indexOf(delimiteur) == -1)
							System.out.println(Messages.getString("FondCarte.13")); //$NON-NLS-1$
						else { // recherche de ligne Region
							while ((ligneMif.length() < 6) || !ligneMif.substring(0, 6).toUpperCase().equals("REGION")) { //$NON-NLS-1$
								ligneMif = tamponMif.readLine();
							}
							centrePresent = false;

							while (ligneMif != null && !(ligneMif.equals(""))) { //$NON-NLS-1$

								//récupération du nombre de polygone par
								// région: dernier caractére de
								//la chaine que l'on a récupéré que l'on
								// transforme en Integer pour ensuite
								// le transformer en int
								ligneMif = ligneMif.substring(6); //Récupération nombre de polygones
								//while (ligneMif.startsWith(" ")) { //$NON-NLS-1$
								//	ligneMif = ligneMif.substring(1);
								//}
								ligneMif = ligneMif.trim();
								nbPoly = Integer.valueOf(ligneMif).intValue();
								//System.out.println("2: " + nbPoly); //debug

								mPoly = new MetaPolygone();

								ligneMif = tamponMif.readLine(); //ligne
								// contenant le
								// nombre de
								// coordonnées
								// par polygone
								for (int i = 0; i < nbPoly; i++) {
									//System.out.println("ligne: " + ligne);
									// //debug

									poly = new Polygone();

									//while (ligneMif.startsWith(" ")) { //$NON-NLS-1$
									//	ligneMif = ligneMif.substring(1);
									//}
									ligneMif = ligneMif.trim();
									nbPoints = (new Integer(ligneMif)).intValue();
									//System.out.println("nombre de Polygone: "
									// + nbPoints); //debug

									for (int j = 0; j < nbPoints; j++) {

										ligneMif = tamponMif.readLine();
										ligneMif = ligneMif.trim();
										positionCar = ligneMif.indexOf(" "); //$NON-NLS-1$

										X = (new Double(ligneMif.substring(0, positionCar))).doubleValue();
										Y = (new Double(ligneMif.substring(positionCar))).doubleValue();

										double[] point = new double[2];
										point[0] = X;
										point[1] = Y;

										poly.ajouterPoint(point);

										//System.out.println("X: " + X + " Y: "
										// +Y ); //debug
									}
									ligneMif = tamponMif.readLine();

									mPoly.ajouterPolygone(poly);
								}

								positionCar = ligneMid.indexOf(delimiteur); //position
								// du
								// premier
								// delimiteur

								// nb de colonnes: taille du Vector données +
								// les deux colonnes nom
								// et geocodes
								nbColonnes = donnees.size();

								//recuperation du geocode
								mPoly.setGeocode(ligneMid.substring(1, positionCar - 1));

								if (nbColonnes > 0) {
									//récupération du Nom de la région
									positionCar2 = ligneMid.indexOf(delimiteur, positionCar + 1);//2e
									// position
									// du
									// délimiteur
									mPoly.setNom(ligneMid.substring(positionCar + 1, positionCar2));
									positionCar = positionCar2;
									//récupération des autres données
									for (int i = 0; i < nbColonnes - 1; i++) {
										//System.out.println("i: " + i +"
										// nbColonnes: " + nbColonnes); //debug
										positionCar2 = ligneMid.indexOf(delimiteur, positionCar + 1);//prochaine
										// position
										// du
										// délimiteur
										((Vector) donnees.get(i)).add(ligneMid.substring(positionCar + 1, positionCar2));
										positionCar = positionCar2;
									}

									((Vector) donnees.get(donnees.size() - 1)).add(ligneMid.substring(positionCar + 1));

								} else
									mPoly.setNom(ligneMid.substring(positionCar + 1));

								ligneMid = tamponMid.readLine();

								//tant qu'on arrive pas é une nouvelle région
								while (ligneMif != null && !(ligneMif.startsWith("Region")) && !(ligneMif.startsWith("REGION")) && !(ligneMif.equals(""))) { //$NON-NLS-1$ //$NON-NLS-2$
									while (ligneMif.startsWith(" ")) {
										ligneMif = ligneMif.substring(1);
									}
									if (ligneMif.substring(0, 6).toUpperCase().equals("CENTER")) { //$NON-NLS-1$
										centrePresent = true;
										positionCar = ligneMif.lastIndexOf(" "); //$NON-NLS-1$
										centreY = (new Double(ligneMif.substring(positionCar + 1))).doubleValue();
										centreX = (new Double(ligneMif.substring(7, positionCar))).doubleValue();
										double[] centroide = new double[2];
										centroide[0] = centreX;
										centroide[1] = centreY;
										mPoly.setCentreOrigine(centroide);
									}

									ligneMif = tamponMif.readLine();
								}
								//Calcul des coordonnées du centroéde
								if (centrePresent == false) {
									Polygone premPoly = (Polygone)mPoly.getEnsPolygones().elementAt(0);
									mPoly.setCentreOrigine(premPoly.getCentroide());
								}
								this.ajouterMetaPoly(mPoly);
							}
							tamponMif.close();
							tamponMid.close();
						}
					}
				}
			}

		} catch (FileNotFoundException e) {
			// Exception déclenchée si le fichier n'existe pas
			System.out.println(Messages.getString("FondCarte.20")); //$NON-NLS-1$
		} catch (EOFException e) {
			// Exception déclenchée si la fin du fichier est atteinte
			System.out.println(Messages.getString("FondCarte.21")); //$NON-NLS-1$
		} catch (IOException e) {
			// Exception déclenchée si un autre probléme survient
			// pendant l'accés au fichier
			System.out.println(Messages.getString("FondCarte.22")); //$NON-NLS-1$
		}

	}

	public boolean isPourMEP() {
		return pourMEP;
	}

	/*
	 * Ouverture du fond de carte au format texte.
	 */
	public void ouvrirFond(String fichier) {
		try {
			FileReader entree = new FileReader(fichier);
			BufferedReader tampon = new BufferedReader(entree);

			String ligne;
			int positionEspace;
			double coordX;
			double coordY;
			double[] point;

			Polygone poly;
			MetaPolygone metaPoly;

			ligne = tampon.readLine();
			//Test de la présence des unités fond(depuis scap-java 0.9)
			if (ligne.startsWith("#")) {
				this.uniteCoord = ligne.substring(1);
				ligne = tampon.readLine();
			}

			// si le fichier ne commence pas par MetaPolygone il n'est pas au
			// bon format
			if (ligne.equals("MetaPolygone")) { //$NON-NLS-1$

				while (ligne != null) {

					metaPoly = new MetaPolygone();

					//récupération du geocode
					ligne = tampon.readLine();
					metaPoly.setGeocode(ligne);

					//recupération du nom
					ligne = tampon.readLine();
					metaPoly.setNom(ligne);

					while (ligne != null && !(ligne.equals("MetaPolygone")) && !(ligne.equals(""))) { //$NON-NLS-1$ //$NON-NLS-2$
						poly = new Polygone();
						ligne = tampon.readLine(); //on passe lit la ligne
						// "Poly"
						ligne = tampon.readLine(); //on passe é la prochaine
						// ligne.
						while (ligne != null && !(ligne.equals("Poly")) && !(ligne.equals("MetaPolygone")) && !(ligne.equals("")) && !(ligne.equals("Centroide"))) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
							// recupération d'une coordonnée
							positionEspace = ligne.indexOf(";"); //$NON-NLS-1$
							point = new double[2];
							//abscisse
							point[0] = (new Double(ligne.substring(0, positionEspace))).doubleValue();
							//ordonnée
							point[1] = (new Double(ligne.substring(positionEspace + 1))).doubleValue();
							poly.ajouterPoint(point);
							ligne = tampon.readLine();
						}
						//ligne = tampon.readLine();
						//while (ligne != null && !(ligne.equals("Poly")) &&
						// !(ligne.equals("MetaPolygone")) &&
						// !(ligne.equals(""))) {
						if (ligne.equals("Centroide")) { //$NON-NLS-1$
							ligne = tampon.readLine();
							positionEspace = ligne.indexOf(";"); //$NON-NLS-1$
							point = new double[2];
							//abscisse
							point[0] = (new Double(ligne.substring(0, positionEspace))).doubleValue();
							//ordonnée
							point[1] = (new Double(ligne.substring(positionEspace + 1))).doubleValue();
							metaPoly.setCentreOrigine(point);
							ligne = tampon.readLine();
						}
						//}
						metaPoly.ajouterPolygone(poly);
						//System.out.println("sortie poly");
					}
					//System.out.println("Sortie MetaPolygone");
					this.ajouterMetaPoly(metaPoly);
				}
				tampon.close();
				//this.afficherCoord(); //debug
			} else
				System.out.println(Messages.getString("FondCarte.36")); //$NON-NLS-1$

		} catch (FileNotFoundException e) {
			// Exception déclenchée si le fichier n'existe pas
			System.out.println(Messages.getString("FondCarte.37")); //$NON-NLS-1$
		} catch (EOFException e) {
			// Exception déclenchée si la fin du fichier est atteinte
			System.out.println(Messages.getString("FondCarte.38")); //$NON-NLS-1$
		} catch (IOException e) {
			// Exception déclenchée si un autre probléme survient
			// pendant l'accés au fichier
			System.out.println(Messages.getString("FondCarte.39")); //$NON-NLS-1$
		}
	}

	/* Méthodes d'accés au champ Carte */
	public void setCarte(Carte c) {
		this.carteEnCours = c;
	}

	public void setCoefMEP(double coefMEP) {
		this.coefMEP = coefMEP;
	}

	/* Méthodes d'accés au champ ensMetaPoly */
	public void setEnsMetaPoly(Vector v) {
		this.ensMetaPoly = v;
	}

	/*
	 * fonction qui permet de modifier les paramétres d'affichage selon la carte
	 * en cours
	 */
	public void setParams() {
		int i = 0;
		int j = 0;
		double valeur = 0;
		double maxi = 0;
		//Iterator index = ensMetaPoly.iterator();
		boolean estNul = false;
		int champNumP = 0;
		int champNumS = 0;
		//if (carteEnCours.getTypeAnalyse() != 0 && carteEnCours.getTypeAnalyse() != 1) {
			champNumS = carteEnCours.analyseSymb.getChampNum();
		//}
		//if (carteEnCours.getTypeAnalyse() != 0 && carteEnCours.getTypeAnalyse() != 2) {
			champNumP = carteEnCours.analysePlages.getChampNum();
		//}
		if (carteEnCours.getTypeAnalyse() != 0 && carteEnCours.getTypeAnalyse() != 1) { //Si analyse en
			// symboles en cours
			if (carteEnCours.analyseSymb.champNum != -1 && carteEnCours.analyseSymb.champNum != PrecedentChamp) setPosDec();
		}
		for (i = 0; i < ensMetaPoly.size(); i++) {
			//Fond blanc
			if (carteEnCours.getTypeAnalyse() == 0) {
				((MetaPolygone) ensMetaPoly.elementAt(i)).getParams().couleur = Color.white;
			}
			//Remplissage valeurs pour ch
			if (champNumP != -1) {
				estNul = (tableau.elementAt((i * colonnes.size()) + champNumP) == null);
				((MetaPolygone) ensMetaPoly.get(i)).getParams().estNul = estNul;
				if (!estNul) {
					switch (((champDonnees) colonnes.get(champNumP)).typeI) {
					case 0:
						valeur = ((Integer) tableau.elementAt((i * colonnes.size()) + champNumP)).intValue();
						break;
					case 2:
						valeur = ((Double) tableau.elementAt((i * colonnes.size()) + champNumP)).doubleValue();
						break;
					}
				}
			}
			if (carteEnCours.getTypeAnalyse() != 0 && carteEnCours.getTypeAnalyse() != 2) { //Si analyse en
				// plages de couleurs
				//estNul = (tableau.elementAt((i * colonnes.size()) + champNumP) == null);
				for (j = 0; j < carteEnCours.analysePlages.nbreClasses; j++) {
					if (j != (carteEnCours.analysePlages.nbreClasses - 1)) {
						maxi = getClasse(j).limiteSup;
					} else {
						maxi = getClasse(j).limiteSup + 1;
					}
					if (estNul) {
						((MetaPolygone) ensMetaPoly.elementAt(i)).getParams().couleur = Color.white;
						break;
					}
					if (valeur >= getClasse(j).limiteInf && valeur < maxi) {
						MetaPolygone MP = (MetaPolygone) ensMetaPoly.elementAt(i);
						MP.getParams().numClasse = j;
						MP.getParams().couleur = getClasse(j).couleur;
						break;
					}
				}
			}
			if (carteEnCours.getTypeAnalyse() != 0 && carteEnCours.getTypeAnalyse() != 1) { //Si analyse en
				// symboles en cours
				double coefSymb = 0;
				double surface = 0;
				double coefType = 0;
				MetaPolygone mPoly;
				mPoly = (MetaPolygone) ensMetaPoly.elementAt(i);
				if (carteEnCours.getTypeAnalyse() == 2) mPoly.getParams().couleur = Color.white;
				switch (carteEnCours.analyseSymb.typeSymbole) {
				case 0:
					coefType = 1;
					break;
				case 1:
					coefType = 1;
				}
				if (tableau.elementAt((i * colonnes.size()) + champNumS) != null) {
					switch (((champDonnees) colonnes.get(champNumS)).typeI) {
					case 0:
						valeur = ((Integer) tableau.elementAt((i * colonnes.size()) + champNumS)).intValue();
						break;
					case 2:
						valeur = ((Double) tableau.elementAt((i * colonnes.size()) + champNumS)).doubleValue();
						break;
					}
				} else {
					valeur = 0;
				}
				if (valeur > carteEnCours.analyseSymb.valVide) {
					mPoly.getParams().estEvide = true;
				} else {
					mPoly.getParams().estEvide = false;
				}
				switch (carteEnCours.analyseSymb.typeProportion) {
				case 0:
					coefSymb = Math.abs(carteEnCours.analyseSymb.surfRef / carteEnCours.analyseSymb.valeurReference);
					surface = Math.abs(valeur) * coefSymb;
					mPoly.getParams().negatif = (valeur < 0);
					mPoly.getParams().tailleSymbole = Math.sqrt(surface / coefType);
					break;
				case 1:
					coefSymb = Math.abs(carteEnCours.analyseSymb.surfRef / (Math.pow(carteEnCours.analyseSymb.valeurReference, (1 / carteEnCours.analyseSymb.facteurRacine))));
					surface = Math.pow(Math.abs(valeur), (1 / carteEnCours.analyseSymb.facteurRacine)) * coefSymb;
					mPoly.getParams().negatif = (valeur < 0);
					mPoly.getParams().tailleSymbole = Math.sqrt(surface / coefType);
					break;
				case 2:
					coefSymb = Math.abs(carteEnCours.analyseSymb.surfRef / Math.log(Math.abs(carteEnCours.analyseSymb.valeurReference)));
					surface = Math.abs(Math.log(Math.abs(valeur))) * coefSymb;
					mPoly.getParams().negatif = (valeur < 0);
					mPoly.getParams().tailleSymbole = Math.sqrt(surface / coefType);
					break;
				}
			}
		}

		PrecedentChamp = champNumS;
	}

	private void setPosDec() {
		int i = 0;
		int ni = tableau.size() / colonnes.size();
		posDec = new int[ni];
		vNul = new boolean[ni];
		double[] r = new double[ni];

		for (i = 0; i < ni; i++) {
			int position = (i * colonnes.size()) + carteEnCours.analyseSymb.champNum;
			if (tableau.elementAt(position) == null) {
				r[i] = Double.MIN_VALUE;
				vNul[i] = true;
			} else {
				vNul[i] = false;
				switch (((champDonnees) colonnes.get(carteEnCours.analyseSymb.champNum)).typeI) {
				case 0:
					r[i] = (((Integer) tableau.elementAt(position)).doubleValue());
					break;
				case 2:
					r[i] = (((Double) tableau.elementAt(position)).doubleValue());
					break;
				}
			}
		}
		triInsertion(r);
	}

	public void setPourMEP(boolean pourMEP) {
		this.pourMEP = pourMEP;
	}

	/* Méthodes d'accés au champ Tableau */
	public void setTableauCols(Vector t, Vector c) {
		this.tableau = t;
		this.colonnes = c;
	}

	public void setUniteCoord(String uniteCoord) {
		this.uniteCoord = uniteCoord;
	}

	private void triBulle(double[] valeurs) {
		int i;
		boolean permut = true;
		double tempVal = 0;
		int tempNdx = 0;

		for (i = 0; i < valeurs.length; i++) {
			posDec[i] = i;
		}
		while (permut == true) {
			permut = false;
			for (i = 0; i < (valeurs.length - 1); i++) {
				if (valeurs[i] < valeurs[i + 1]) {
					tempVal = valeurs[i];
					valeurs[i] = valeurs[i + 1];
					valeurs[i + 1] = Math.abs(tempVal);
					tempNdx = posDec[i];
					posDec[i] = posDec[i + 1];
					posDec[i + 1] = tempNdx;
					permut = true;
				}
			}
		}
	}

	private void triInsertion(double[] a) {
		int i;
		int l = a.length;
		for (i = 0; i < l; i++) {
			posDec[i] = i;
		}
		for (i = 1; i < l; i++) {
			double temp = a[i];
			int tx = posDec[i];
			int j;
			for (j = i; j > 0 && a[j - 1] < temp; j--) {
				a[j] = a[j - 1];
				posDec[j] = posDec[j - 1];
			}
			a[j] = temp;
			posDec[j] = tx;
		}
	}

	private void triShell(double[] valeurs) {
		int d = 1;
		int j;
		int n = valeurs.length;
		while (9 * d < n)
			d = 3 * d + 1;
		while (d > 0)  {
			for (int i = d; i < n; i++) {
				double temp = valeurs[i];
				j = i;
				while (j >= d && valeurs[j - d] > temp) {
					valeurs[j] = valeurs[j - d];
					j -= d;
				}
				valeurs[j] = temp;
			}
			d /= 3;
		}
	}

	/*
	 * fonction qui va renvoyer un Vector de 4 éléments contenant les
	 * coordonnées extrémes de la carte le vector renvoyé contiendra : en
	 * position 0, la coordonnée la plus é gauche (x) en position 1, la
	 * coordonnée la plus en haut (y) en position 2, la coordonnée la plus é
	 * droite (x) en position 3, la coordonnée la plus en bas (y) met dans Xmin
	 * et Ymax l'abscisse la plus petite et l'ordonnée la plus grande
	 */
	public double[] trouverExtremes() {

		double[] extremesa;
		double[] extremesCourant;

		extremesa = ((MetaPolygone) this.ensMetaPoly.get(0)).trouverExtremes();
		this.nbPts = ((MetaPolygone) this.ensMetaPoly.get(0)).getNbPts();

		for (int i = 1; i < this.ensMetaPoly.size(); i++) {
			extremesCourant = ((MetaPolygone) this.ensMetaPoly.get(i)).trouverExtremes();
			this.nbPts += ((MetaPolygone) this.ensMetaPoly.get(i)).getNbPts();

			if (extremesCourant[0] < extremesa[0])
				extremesa[0] = extremesCourant[0];
			if (extremesCourant[1] > extremesa[1])
				extremesa[1] = extremesCourant[1];
			if (extremesCourant[2] > extremesa[2])
				extremesa[2] = extremesCourant[2];
			if (extremesCourant[3] < extremesa[3])
				extremesa[3] = extremesCourant[3];
		}

		this.Xmin = extremesa[0];
		this.Ymax = extremesa[1];

		return extremesa;
	}

	public double[] trouverLimites() {
		double[] extremesa;
		double[] extremesCourant;
		extremesa = ((MetaPolygone)this.ensMetaPoly.get(0)).trouverLimites();
		for (int i = 1; i < this.ensMetaPoly.size(); i++) {
			extremesCourant = ((MetaPolygone)this.ensMetaPoly.get(1)).trouverLimites();
			if (extremesCourant[0] < extremesa[0])
				extremesa[0] = extremesCourant[0];
			if (extremesCourant[1] > extremesa[1])
				extremesa[1] = extremesCourant[1];
			if (extremesCourant[2] > extremesa[2])
				extremesa[2] = extremesCourant[2];
			if (extremesCourant[3] < extremesa[3])
				extremesa[3] = extremesCourant[3];
		}
		
		return extremesa;
	}

	public double getXmin() {
		return Xmin;
	}

	public void setXmin(double xmin) {
		Xmin = xmin;
	}

	public double getYmax() {
		return Ymax;
	}

	public void setYmax(double ymax) {
		Ymax = ymax;
	}

	public int getPSel() {
		return pSel;
	}

	public void setPSel(int sel) {
		pSel = sel;
	}

	public int getNbpts() {
		return this.nbPts;
	}

	public double[] getExtremes() {
		return extremes;
	}
	
	public void testJTS() {
		MetaPolygone mp0 = (MetaPolygone) this.ensMetaPoly.get(0);
		Polygone p0 = (Polygone) mp0.getEnsPolygones().get(0);
		MetaPolygone mp1 = (MetaPolygone) this.ensMetaPoly.get(1);
		Polygone p1 = (Polygone) mp1.getEnsPolygones().get(0);
		Integer nbPts0 = p0.getNbPts();
		Coordinate[] coordonnees0 = new Coordinate[nbPts0];
		for (int i = 0; i < nbPts0; i++) {
			coordonnees0[i] = new Coordinate(p0.xpoints[i], p0.ypoints[i]);
		}
		Geometry g0 = new GeometryFactory().createLineString(coordonnees0);
		System.out.println("Geometry 0 : " + g0);
		
		Integer nbPts1 = p1.getNbPts();
		Coordinate[] coordonnees1 = new Coordinate[nbPts1];
		for (int i = 0; i < nbPts1; i++) {
			coordonnees1[i] = new Coordinate(p1.xpoints[i], p1.ypoints[i]);
		}
		Geometry g1 = new GeometryFactory().createLineString(coordonnees1);
		System.out.println("Geometry 1 : " + g1);
		
		Geometry g3 = g1.buffer(20,16);
		System.out.println("Geometry 3 : " + g3);
		
		Polygone p3 = new Polygone();
		Polygon pg3 = (Polygon) g3;
		LineString ls3 = pg3.getExteriorRing();
		Coordinate[] co = pg3.getCoordinates();
		
		for (int i = 0; i < g3.getNumPoints(); i++) {
			p3.addPoint((int) co[i].x, (int) co[i].y);
		}
		MetaPolygone mp3 = new MetaPolygone();
		mp3.ajouterPolygone(p3);
		this.ensMetaPoly.add(mp3);
		this.setParams();
	}
	
}

