/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package carto;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.io.BufferedWriter;
import java.io.Serializable;
import java.util.Vector;

public class MetaPolygone implements Cloneable, Serializable {

    private double[] centreOrigine;

    //par ex la couleur du MetaPolygone, la taille du symbole qui
    //est dessiné. Stocké independamment pour pouvoir l'enregistrer
    //plus facilement.Comme cela on la distingue du fond de carte
    private Point centroide;

    private Vector ensPolygones; //Vector contenant les polygones contenus par le MetaPolygone
    private String geocode;

    private String nom;
    
    private int nbPts = 0;

    protected ParamsPolygone params; //contient tout ce qui concerne l'analyse par
                               // MetaPolygone

    /** Creates a new instance of MetaPolygone */
    public MetaPolygone() {
        this.ensPolygones = new Vector();
        this.centreOrigine = new double[2];
        this.params = new ParamsPolygone();
    }

    /* Méthodes d'accés é l'attribut centre */

    /*
     * fonction de debuggage : affichage des coordonnées d'un MetaPolygone
     */
    public void afficherCoord() {
        Polygone poly;
        System.out.println(Messages.getString("MetaPolygone.1")); //$NON-NLS-1$
        if (this.ensPolygones != null) {
            for (int i = 0; i < this.ensPolygones.size(); i++) {
                poly = (Polygone) this.ensPolygones.elementAt(i);
                poly.afficherCoord();
            }
        } else {
            System.out.println(Messages.getString("MetaPolygone.2")); //$NON-NLS-1$
        }
    }

    /* fonction qui permet d'ajouter un polygone au MetaPolygone */
    public void ajouterPolygone(Polygone p) {
        this.ensPolygones.addElement(p);
    }

    /*
     * fonction qui teste si le point p appartient au MetaPolygone en cours
     */
    public boolean appartenir(Point2D p) {
        int i = 0;
        boolean appartient = false;
        while (i < this.ensPolygones.size() && appartient == false) {
            appartient = ((Polygone) this.ensPolygones.get(i)).contains(p);
            i++;
        }
        return appartient;
    }
    /*
     * Fonction d'aggrandissement du MetaPolygone
     */
    public void calculerCoord(int larg, int haut, double coeff) {
        Polygone poly;
        this.centroide = new Point();
        this.centroide.x = (int) (this.centreOrigine[0] * coeff) + larg;
        this.centroide.y = (int) -(this.centreOrigine[1] * coeff) + haut;
        for (int i = 0; i < this.ensPolygones.size(); i++) {
            ((Polygone) this.ensPolygones.elementAt(i)).calculerCoord(larg, haut, coeff);
        }
    }
    
    /* Méthodes d'accés é l'attribut ensPolygone */

    /* Redéfinition de la méthode clone */
    public Object clone() {
        try {
            MetaPolygone mPoly = (MetaPolygone) super.clone();
            mPoly.ensPolygones = new Vector();
            if (this.ensPolygones != null) {
                for (int i = 0; i < this.ensPolygones.size(); i++) {
                    mPoly.ensPolygones.add(i, (Polygone) ((Polygone) this.ensPolygones.elementAt(i)).clone());
                }
            }
            return mPoly;
        } catch (CloneNotSupportedException x) {
            return (new InternalError(Messages.getString("MetaPolygone.0"))); //$NON-NLS-1$
        }

    }

    /*
     * fonction qui permet d'adapter les coordonnées récupérées d'un format MIF
     * pour les adapter au systéme de coordonnées Java
     */
    public void decalerCoord(double Xmin, double Ymax) {
        Polygone poly;
        this.centreOrigine[0] = this.centreOrigine[0] - Xmin;
        this.centreOrigine[1] = this.centreOrigine[1] - Ymax;
        for (int i = 0; i < this.ensPolygones.size(); i++) {
            poly = (Polygone) this.ensPolygones.get(i);
            poly.decalerCoord(Xmin, Ymax);
            this.ensPolygones.setElementAt(poly, i);
        }
    }

    /* Méthodes d'accés é l'attribut Géocode */

    /* Methode de dessin du MetaPolygone */
    public void drawMetaPolygone(Graphics g, boolean type4) {
        Polygone poly;
        Color couleur;
        for (int i = 0; i < this.ensPolygones.size(); i++) {
            poly = ((Polygone) this.ensPolygones.elementAt(i));
            if (this.params.estNul || type4) {
                couleur = Color.white;
            } else {
                couleur = this.params.couleur;                
            }
            if (!(couleur.equals(Color.white))) {
                g.setColor(couleur);
                g.fillPolygon(poly);
            }
            g.setColor(Color.gray);
            g.drawPolygon(poly);
        }
    }

    /*
     * enregistrement d'un MetaPolygone au format texte
     */
    public void enregistrerMP(BufferedWriter tampon) {
        try {
            tampon.write("MetaPolygone"); //$NON-NLS-1$
            tampon.newLine();
            tampon.write(this.geocode);
            tampon.newLine();
            tampon.write(this.nom);
            tampon.newLine();
            for (int i = 0; i < this.ensPolygones.size(); i++) {
                tampon.write("Poly"); //$NON-NLS-1$
                tampon.newLine();
                ((Polygone) this.ensPolygones.get(i)).enregistrerPoly(tampon);
            }
            tampon.write("Centroide"); //$NON-NLS-1$
            tampon.newLine();
            tampon.write(String.valueOf(this.centreOrigine[0]) + ";" + String.valueOf(this.centreOrigine[1])); //$NON-NLS-1$
            tampon.newLine();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* Méthodes d'accés é l'attribut Nom */

    public double[] getCentreOrigine() {
        return this.centreOrigine;
    }

    public Point getCentroide() {
        return centroide;
    }

    /* Méthodes d'accés é l'attribut Params */

    public Vector getEnsPolygones() {
        return this.ensPolygones;
    }

    public String getGeocode() {
        return this.geocode;
    }

    
    public String getNom() {
        return this.nom;
    }

    public ParamsPolygone getParams() {
        return this.params;
    }

    public void setCentreOrigine(double[] p) {
        this.centreOrigine = p;
    }

    public void setCentroide(Point centroide) {
        this.centroide = centroide;
    }

    public void setEnsPolygones(Vector v) {
        this.ensPolygones = v;
    }

    public void setGeocode(String g) {
        this.geocode = g;
    }

    public void setNom(String n) {
        this.nom = n;
    }

    public void setParams(ParamsPolygone p) {
        this.params = p;
    }

    /*
     * Fonction qui permet de trouver les coordonnées extremes du MetaPolygone
     */
    public double[] trouverExtremes() {
        double[] extremes; // tableau des coordonnées extremes renvoyé
        double[] extremesCourant; //tableau des coordonnées extremes du
                                  // polygone courant
        extremes = ((Polygone) this.ensPolygones.get(0)).trouverExtremes();
        this.nbPts = ((Polygone) this.ensPolygones.get(0)).getNbPts();

        for (int i = 1; i < this.ensPolygones.size(); i++) {
            extremesCourant = ((Polygone) this.ensPolygones.get(i)).trouverExtremes();
            this.nbPts += ((Polygone) this.ensPolygones.get(i)).getNbPts();
            
            if (extremesCourant[0] < extremes[0])
                extremes[0] = extremesCourant[0];
            if (extremesCourant[1] > extremes[1])
                extremes[1] = extremesCourant[1];
            if (extremesCourant[2] > extremes[2])
                extremes[2] = extremesCourant[2];
            if (extremesCourant[3] < extremes[3])
                extremes[3] = extremesCourant[3];
        }
        return extremes;
    }
    
    public double[] trouverLimites() {
    	double[] extremes;
    	double[] extremesCourant;
    	extremes = ((Polygone)this.ensPolygones.get(0)).trouverLimites();
    	for (int i = 1; i < this.ensPolygones.size(); i++) {
    		 extremesCourant = ((Polygone)this.ensPolygones.get(1)).trouverLimites();
             if (extremesCourant[0] < extremes[0])
                 extremes[0] = extremesCourant[0];
             if (extremesCourant[1] > extremes[1])
                 extremes[1] = extremesCourant[1];
             if (extremesCourant[2] > extremes[2])
                 extremes[2] = extremesCourant[2];
             if (extremesCourant[3] < extremes[3])
                 extremes[3] = extremesCourant[3];
    	}
    	return extremes;
    }
    
    public int getNbPts() {
    	return this.nbPts;
    }

}