/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package guiInterface;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.util.AbstractList;
import java.util.Vector;

/**
 * @author Laurent
 *  
 */
public class GraphicObjectsCollection extends AbstractList {

    protected int m_horizRes = 72;

    protected int m_vertRes = 72;

    private Vector go;

    public GraphicObjectsCollection() {
        //Constructeur vide.
        go = new Vector();
    }

    public GraphicObjectsCollection(Vector a) {
        go = a;
    }

    public Object get(int index) {
        return go.elementAt(index);
    }

    public Object set(int index, GraphicObject o) {
        GraphicObject oldValue = (GraphicObject) go.elementAt(index);
        go.setElementAt(o, index);
        return oldValue;
    }

    public int size() {
        return go.size();
    }

    public void add(GraphicObject o) {
        go.add(o);
    }

    public void remove(GraphicObject o) {
        go.remove(o);
    }

    /*
     * public boolean contains (GraphicObject o) { return this.contains(o); }
     * 
     * public int indexOf(GraphicObject o) { return this.indexOf(o); }
     */

    public void moveUp(GraphicObject o) {
        int classement = this.indexOf(o);
        if (classement == this.size())
            return;
        this.remove(o);
        this.add(classement + 1, o);
    }

    public void moveDown(GraphicObject o) {
        int classement = this.indexOf(o);
        if (classement == 0)
            return;
        this.remove(o);
        this.add(classement - 1, o);
    }

    public void moveAllUp(GraphicObject o) {
        int classement = this.indexOf(o);
        if (classement == this.size())
            return;
        this.remove(o);
        this.add(this.size(), o);
    }

    public void moveAllDown(GraphicObject o) {
        int classement = this.indexOf(o);
        if (classement == 0)
            return;
        this.remove(o);
        this.add(0, o);
    }

    public void drawSelectedObject(Graphics g, GraphicObject sO, double scale) {
        Graphics2D g2 = (Graphics2D) g;
        g2.scale(scale, scale);

        if (sO != null) {
            Stroke originePen = g2.getStroke();
            BasicStroke selectionPen = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, new float[] { 6.0f, 2.0f }, 0.0f);
            if (sO.getRotation() != 0) {
                g2.rotate(sO.getRotation());
            }

            g2.setStroke(selectionPen);
            g2.setPaint(Color.BLUE);

            /*
             * if (sO.getWidth() < 0) { if (sO.getHeight() < 0) {
             * g2.drawRect(sO.getX() - 2 + sO.getWidth(), sO.getY() - 2 +
             * sO.getHeight(), -sO.getWidth() + 4, -sO.getHeight() + 4); } else {
             * g2.drawRect(sO.getX() - 2 + sO.getWidth(), sO.getY() - 2,
             * -sO.getWidth() + 4, sO.getHeight() + 4); } } else { if
             * (sO.getHeight() < 0) { g2.drawRect(sO.getX() - 2, sO.getY() - 2 +
             * sO.getHeight(), sO.getWidth() + 4, -sO.getHeight() + 4); } else {
             * //g2.drawRect(sO.getX() - 2, sO.getY() - sO.getHeight(),
             * sO.getWidth() + 4, sO.getHeight() + 4);
             */
            g2.drawRect(sO.getX() - 2, sO.getY() - 2, sO.getWidth() + 4, sO.getHeight() + 4);
            //}
            //}
            g2.setStroke(originePen);
        }
    }

    public void drawObjects(Graphics g, double scale) {
        int i;
        Graphics2D g2 = (Graphics2D) g;
        AffineTransform oatx = g2.getTransform();
        g2.scale(scale, scale);

        for (i = 0; i < this.size(); i++) {
            ((GraphicObject) this.get(i)).draw(g);
        }
        g2.setTransform(oatx);
    }

    public void printObjects(Graphics g) {
        int i;

        for (i = 0; i < this.size(); i++) {
            ((GraphicObject) this.get(i)).draw(g);
        }
    }

    public void drawBitmap(Graphics g, double scale) {
        int i;
        Graphics2D g2 = (Graphics2D) g;
        AffineTransform oatx = g2.getTransform();
        g2.scale(scale, scale);
        g.clearRect(0, 0, g.getClipBounds().width, g.getClipBounds().height);

        for (i = 0; i < this.size(); i++) {
            ((GraphicObject) this.get(i)).draw(g);
        }
        g2.setTransform(oatx);
    }

    public void exportSVG() {
        //Reste é coder !
    }

    public GraphicObject findObjectAtPoint(Point pt) {
        int i = 0;

        for (i = go.size() - 1; i >= 0; i--) { //recherche en ordre inverse pour que le plus haut des objets sorte en premier.
            GraphicObject io = (GraphicObject) this.get(i);
            if (io.hitTest(pt))
                return io;
        }
        return null;
    }

    /**
     * @return Returns the m_horizRes.
     */
    public int getHorizRes() {
        return m_horizRes;
    }

    /**
     * @param res
     *            The m_horizRes to set.
     */
    public void setHorizRes(int res) {
        m_horizRes = res;
    }

    /**
     * @return Returns the m_vertRes.
     */
    public int getVertRes() {
        return m_vertRes;
    }

    /**
     * @param res
     *            The m_vertRes to set.
     */
    public void setVertRes(int res) {
        m_vertRes = res;
    }
}